import asyncio
from threading import Thread

from websockets.legacy.server import WebSocketServerProtocol, serve


class WebSocketServer:
    host: str
    port: int
    clients: set[WebSocketServerProtocol]
    loop: asyncio.AbstractEventLoop | None
    thread: Thread | None

    def __init__(self, host: str = "localhost", port: int = 8765):
        self.host = host
        self.port = port
        self.clients = set()
        self.loop = None
        self.thread = None

    async def _handler(self, ws: WebSocketServerProtocol):
        self.clients.add(ws)
        try:
            await ws.wait_closed()
        finally:
            self.clients.remove(ws)

    def start(self):
        def run():
            self.loop = asyncio.new_event_loop()
            asyncio.set_event_loop(self.loop)
            ws_server = serve(
                ws_handler=self._handler, host=self.host, port=self.port
            )
            self.loop.run_until_complete(ws_server)
            self.loop.run_forever()

        self.thread = Thread(target=run, daemon=True)
        self.thread.start()

    async def _broadcast(self, message: str):
        for ws in self.clients.copy():
            try:
                await ws.send(message)
            except Exception:
                self.clients.discard(ws)

    def notify_all(self, message: str = "reload"):
        if self.loop and self.clients:
            asyncio.run_coroutine_threadsafe(
                coro=self._broadcast(message), loop=self.loop
            )
